package cc.smtweb.system.bpm.web.engine.flow;

import cc.smtweb.framework.core.db.DbEngine;
import cc.smtweb.framework.core.db.EntityHelper;
import cc.smtweb.framework.core.exception.BizException;
import cc.smtweb.framework.core.session.UserSession;
import cc.smtweb.framework.core.util.DateUtil;
import cc.smtweb.framework.core.util.NumberUtil;
import cc.smtweb.system.bpm.web.design.flow.FlowConst;
import cc.smtweb.system.bpm.web.design.flow.ModelProc;
import cc.smtweb.system.bpm.web.design.flow.ModelProcHelper;
import cc.smtweb.system.bpm.web.design.flow.define.Activity;
import cc.smtweb.system.bpm.web.engine.flow.define.ProcinstEntity;
import cc.smtweb.system.bpm.web.engine.flow.define.TaskEntity;
import cc.smtweb.system.bpm.web.sys.base.dict.DictCache;
import cc.smtweb.system.bpm.web.sys.user.dept.Dept;
import cc.smtweb.system.bpm.web.sys.user.dept.DeptCache;
import cc.smtweb.system.bpm.web.sys.user.party.Party;
import cc.smtweb.system.bpm.web.sys.user.party.PartyCache;
import org.apache.commons.lang3.StringUtils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Created by Akmm at 2020/3/4 11:35
 * 单据流程实例组件
 */
public class FlowInstance {
    //会话信息
    private UserSession us;
    //流程定义
    public ModelProc proc_def;

    //流程实例
    public ProcinstEntity proc_inst;

    //活动任务
    public TaskEntity act_inst;

    //操作类型 新增/编辑/查看
    public int opt_mode = FlowConst.OperatorType.VIEW;

    //流程执行变量
//    public Map<String, String> variables = new HashMap<>();

    //当前任务
    public List<Map<String, String>> tasks = new ArrayList<>();

    public FlowInstance(UserSession us) {
        this.us = us;
    }

    //能否编辑
    public boolean canEdit() {
        //新增状态可以编辑
        if (opt_mode == FlowConst.OperatorType.NEW || proc_inst.getStatu() == FlowConst.InstanceStatu.BEGIN.value)
            return true;
        return false;
    }

    /**
     * 流程条件
     *
     * @param expr
     * @return
     */
    public boolean execTrans(String expr) {
        if (StringUtils.isBlank(expr)) return true;
        Map<String, Object> mapVar = new HashMap<>();
        /*mapVar.putAll(billEntity.getData());
        mapVar.put("party_code", PartyEntityBuffer.getInstance().getCodeById(billEntity.getPartyId()));
        mapVar.put("party_name", PartyEntityBuffer.getInstance().getNameById(billEntity.getPartyId()));
        mapVar.put("dept_code", DepartmentEntityBuffer.getInstance().getCodeById(billEntity.getDeptId()));
        mapVar.put("dept_name", DepartmentEntityBuffer.getInstance().getNameById(billEntity.getDeptId()));
        mapVar.put("bill_user_code", UserBaseEntityBuffer.getInstance().getCodeById(billEntity.getBillUser()));
        mapVar.put("bill_user_name", UserBaseEntityBuffer.getInstance().getNameById(billEntity.getBillUser()));
        localBuildExecTrans(mapVar);*/
        Object o = NumberUtil.calcExprMapObject(expr, mapVar);
        return o != null && (Boolean) o;
    }

    /**
     * 新建流程实例
     *
     * @param bill_type
     * @throws Exception
     */
    public void create(int bill_type) throws Exception {
        Party loginParty = PartyCache.getInstance().get(us.getPartyId());
        if (EntityHelper.isEmpty(loginParty)) {
            throw new BizException("当前登录人员管理单位信息为空，请检查后保存！");
        }

        Dept loginDept = DeptCache.getInstance().get(us.getDeptId());
        if (EntityHelper.isEmpty(loginDept)) {
            loginDept = new Dept();
        }

        proc_def = ModelProcHelper.getBillProc(bill_type, us.getUserId()); //根据单据分类获取流程定义
        if (EntityHelper.isEmpty(proc_def)) {
//            throw new BizException("没有找到对应的流程定义，请到【系统设置】-【单据流程分配】为当前单据类型【" + BillTypeCache.getInstance().getName(bill_type) + "】分配工作流程！");
        }

        Activity start = ModelProcHelper.getFirstActivity(proc_def.getEntityId());
        if (start == null) {
            throw new BizException("对应的流程没有定义有效的活动步骤！");
        }

        opt_mode = FlowConst.OperatorType.NEW;

        proc_inst = new ProcinstEntity();
        proc_inst.init();
        proc_inst.setEntityId(DbEngine.getInstance().nextId());
        proc_inst.setPartyId(loginParty.getEntityId());
        proc_inst.setDeptId(loginDept.getEntityId());
        proc_inst.setBillType(bill_type);
        proc_inst.setMakeDate(DateUtil.nowDateTimeLong());
        proc_inst.setPrcId(proc_def.getEntityId());
        proc_inst.setStartTime(DateUtil.nowDateTimeLong());
        proc_inst.setUserId(us.getUserId());
        proc_inst.setStatu(FlowConst.InstanceStatu.BEGIN.value);

        act_inst = new TaskEntity();
        act_inst.init();
        act_inst.setEntityId(DbEngine.getInstance().nextId());
        act_inst.setId(proc_inst.getEntityId());
        act_inst.setPrcId(proc_def.getEntityId());
        act_inst.setActId(start.getId());
        act_inst.setHandler(us.getUserId());
        act_inst.setStartTime(DateUtil.nowDateTimeLong());
        act_inst.setStatu(FlowConst.ActivityStatu.HANDLE.value);
        act_inst.setMake(true);
        act_inst.setSign(false);
        act_inst.setReject(false);
        act_inst.setRetake(false);

        proc_inst.setTaskId(act_inst.getEntityId());
    }

    /**
     * 办理
     *
     * @throws Exception
     */
    public void handler() {
        long user_id = us.getUserId();
        //校验权限，当前人员有权限干不，没权限直接抛异常
//        checkPermission();
        if (act_inst.getStatu() != FlowConst.ActivityStatu.WAIT.value) {
            throw new BizException("当前任务非待办状态，不能办理失败！");
        }

        act_inst.setHandler(user_id);
        act_inst.setStartTime(DateUtil.nowDateTimeLong());
        act_inst.setStatu(FlowConst.ActivityStatu.HANDLE.value);

        proc_inst.setTaskId(act_inst.getEntityId());
        proc_inst.setStatu(FlowConst.InstanceStatu.RUNING.value);

        DbEngine.getInstance().findDao(TaskEntity.ENTITY_NAME).updateEntity(act_inst);
        DbEngine.getInstance().findDao(ProcinstEntity.ENTITY_NAME).updateEntity(proc_inst);

        buildBillLog(FlowConst.Button.HANDLE.value, act_inst, null);
//        comp.transCallback(CmEnum.FlowOptType.RETAKE.value, act_inst, null);
    }

    /**
     * 状态变更，构建日志
     */
    protected void buildBillLog(final String flow_opt, TaskEntity srcTask, List<TaskEntity> dstTasks) {
        /*LogEntity logEntity = new LogEntity();
        logEntity.setSuggestion(context.getDfpRequest().getParams().getStrIgnoreNull("suggestion"));
        logEntity.setFlowOpt(flow_opt);

        logEntity.setBillId(proc_inst.getBillId());
        logEntity.setOptTime(UtilPub.getLastTime());

        UserBaseEntity user = UtilLogin.getLoginUser(context.getLoginInfo());
        logEntity.setUserId(user.getEntityId());
        logEntity.setUserName(user.getName());
        logEntity.setLogId(PKGenerator.newId());
        logEntity.setSignPicture(UserBaseEntityBuffer.getInstance().getUserCardEntity(user.getUserId()).getSignPicture());

        String src = null, dst = null;
        String step_name = null;
        StringBuilder sbDst = null;
        if (srcTask != null) {
            src = ActivityEntityBuffer.getInstance().getNameById(srcTask.getActId());
            step_name = ActivityEntityBuffer.getInstance().getStepName(srcTask.getActId());
        }
        if (UtilPub.isNotEmpty(dstTasks)) {
            sbDst = new StringBuilder();
            for (ActinstEntity a : dstTasks) {
                sbDst.append("/").append(ActivityEntityBuffer.getInstance().getNameById(a.getActId()));
            }
            dst = sbDst.substring(1);
        }
        if (flow_opt == CmEnum.FlowOptType.RETAKE.value) {
            logEntity.setInfo(dst + "->" + src);
            logEntity.setStepName(step_name);
        } else if (dst != null) {
            logEntity.setInfo(src + "->" + dst);
            logEntity.setStepName(step_name);
        } else if (src != null) {
            logEntity.setInfo(src);
            logEntity.setStepName(step_name);
        } else {
            logEntity.setInfo("-");
        }
        new LogDao().insert(logEntity);
         */
    }
}
