package cc.smtweb.system.bpm.web.login;

import cc.smtweb.framework.core.annotation.SwBody;
import cc.smtweb.framework.core.annotation.SwParam;
import cc.smtweb.framework.core.annotation.SwPerm;
import cc.smtweb.framework.core.annotation.SwService;
import cc.smtweb.framework.core.common.R;
import cc.smtweb.framework.core.common.SwMap;
import cc.smtweb.framework.core.db.DbEngine;
import cc.smtweb.framework.core.exception.BizException;
import cc.smtweb.framework.core.session.SessionManager;
import cc.smtweb.framework.core.session.UserSession;
import cc.smtweb.framework.core.util.PubUtil;
import cc.smtweb.system.bpm.web.sys.user.party.Party;
import cc.smtweb.system.bpm.web.sys.user.party.PartyCache;
import cc.smtweb.system.bpm.web.sys.user.role.RoleCache;
import cc.smtweb.system.bpm.web.sys.user.user.User;
import cc.smtweb.system.bpm.web.sys.user.user.UserCache;
import cc.smtweb.system.bpm.web.sys.user.user.UserRoleCache;
import lombok.extern.slf4j.Slf4j;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

@Slf4j
@SwService
public class AuthService {
    @SwParam
    private DbEngine dbEngine;

    @SwParam
    private SessionManager sessionManager;

    @SwPerm()
    public R getParty(@SwParam("username") String username) {
        Set<Party> partySet = PartyCache.getInstance().getTopSet();
        List<SwMap> list = new ArrayList<>();
        for (Party party : partySet) {
            SwMap data = new SwMap();
            data.put("id", party.getId());
            data.put("text", party.getName());
            data.put("isLeaf", true);
            data.put("children", new ArrayList<>());
            list.add(data);
        }
        return R.success(list);
    }

    @SwPerm()
    public R userInfo(@SwBody SwMap params, UserSession us) {
        SwMap data = new SwMap();
        try {
            User user = (User) UserCache.getInstance().get(us.getUserId()).clone();
            user.setId(us.getUserId());
            user.put("create_time", PubUtil.checkLastTime(user.getCreate()));
            user.put("sur_party", PartyCache.getInstance().get(us.getPartyId()));
            user.put("sur_roles", RoleCache.getInstance().getNamesByIds(UserRoleCache.getInstance().getRoleIdByUP(us.getUserId(), us.getPartyId())));
            data.put("userInfo", user);
        } catch (CloneNotSupportedException e) {
            data.put("userInfo", new SwMap());
        }
        return R.success(data);
    }

    @SwPerm()
    public R saveUser(@SwBody SwMap params, UserSession us) {
        SwMap userInfo = params.readMap("userInfo");
        User user = UserCache.getInstance().get(us.getUserId());
        user.getData().putAll(userInfo);
        DbEngine.getInstance().findDao(User.ENTITY_NAME).updateEntity(user);
        UserCache.getInstance().put(user);
        return R.success();
    }

    @SwPerm()
    public R changePwd(@SwBody SwMap params, UserSession us) {
        String old_pwd = params.readString("old_pwd");
        String new_pwd = params.readString("new_pwd");
        User user = UserCache.getInstance().get(us.getUserId());
        if (!LoginHelper.verifyPwd(user, old_pwd)) {
            return R.error("旧密码错误");
        }
        LoginHelper.resetUserPwd(user, new_pwd);
        return R.success();
    }

    @SwPerm(SwPerm.NONE)
    public R login(@SwBody LoginVO loginPO) {
        SwMap data = new SwMap();
        UserSession userSession = null;
        try {
            userSession = LoginHelper.login(loginPO);
        } catch (BizException e) {
            data.put("isOk", false);
            data.put("msg", e.getMessage());
            return R.success(data);
        } catch (Exception e) {
            data.put("isOk", false);
            data.put("msg", "系统错误");
            return R.success(data);
        }
        userSession.setSiteId(1);
        User user = UserCache.getInstance().get(userSession.getUserId());
        if (user == null) {
            if (1 != userSession.getUserId()) {
                data.put("isOk", false);
                data.put("msg", "账户异常");
                return R.success(data);
            }
            user = new User();
            user.setId(1);
            user.setName("系统管理员");
        }
        LoginAckVO loginAckVO = new LoginAckVO();
        loginAckVO.setUserId(user.getId());
        loginAckVO.setUserName(user.getName());
        loginAckVO.setRealName(user.getName());
        loginAckVO.setPartyId(userSession.getPartyId());
        loginAckVO.setAvatar("");
        loginAckVO.setDesc(user.getSign());
        loginAckVO.setToken(sessionManager.login(userSession));

        data.put("user", loginAckVO);
        data.put("token", loginAckVO.getToken());
        data.put("isOk", true);
        data.put("msg", "");
        return R.success(data);
    }

    @SwPerm()
    public R ping(@SwParam("msg") String msg) {
        return R.success(msg);
    }

    @SwPerm("user:edit")
    public R config(@SwParam("username") String username) {
        return R.success("config: " + username);
    }


    /**
     * 退出登录
     *
     * @return code
     */
    public R logout() {
        sessionManager.logout();
        return R.success();
    }

    // defaultRun 命名的函数是默认函数
//  @SwPerm(SwPerm.NONE)
//  public R defaultRun(@SwPathParam String path) {
//    return R.success(path).put("dao", authDao);
//  }

}
